<?php

require_once('lib-mbapi/include/modules/registrar/registrar.php');

$className = 'HEXONET';
$GLOBALS["moduleInfo"][$className] = array(
	'type'         => 'registrar',
	'status'       => Registrar::STATUS_STABLE,
	'version'      => '1.0.0',
	'displayName'  => 'HEXONET',
	'author'       => 'HEXONET GmbH',
	'file'         => __FILE__,
	'countryCodes' => array('DE', 'CA'),
);

class HEXONET extends Registrar {

// ---------------------------------------------------------------------------------------------------- MODULE VARS
// ---------------------------------------------------------------------------------------------------- MODULE VARS
	public $_debugMode = false;

	public $_contactIDs = array();

	protected $_call_errors = array();

	public function getModuleInfo() {
		return $GLOBALS['moduleInfo']['HEXONET'];
	}

	public function __construct($input) {

		// quick fix to enable IDN support across all functions
		if (isset($input['domainSLD']) && trim($input['domainSLD']) != '') {
		    $input['domainSLD'] = $this->getPUNYCode($input);
		}

		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 1;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 1;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 1;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 1;
		$this->capabilities[REGISTRAR_LIVERENEW] = 1;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 0;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 1;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 0;

		// the following are types of contacts allowed by eNom
		$this->contactTypesMap = array(
			'registrant' => 'OWNERCONTACT',
			'admin' => 'ADMINCONTACT',
			'tech' => 'TECHCONTACT',
			'auxBilling' => 'BILLINGCONTACT'
		);

		// the following contact information is allowed on most actions
		$this->contactParamsMap = array(
			'OrganizationName' => 'ORGANIZATION',
			'FirstName' => 'FIRSTNAME',
			'LastName' => 'LASTNAME',
			'JobTitle' => 'TITLE',
			'Address1' => 'STREET0',
			'Address2' => 'STREET1',
			'City' => 'CITY',
			'State' => 'STATE',
			'PostalCode' => 'ZIP',
			'Country' => 'COUNTRY',
			'Phone' => 'PHONE',
			'Fax' => 'FAX',
			'Email' => 'EMAIL',
		);

		parent::Registrar($input);
	}


// ---------------------------------------------------------------------------------------------------- INTERNAL METHODS
// ---------------------------------------------------------------------------------------------------- INTERNAL METHODS



	/**
	 * This function return the HEXONET/ISPAPI connection URL
	 */
	protected function getURL() {
		return "https://coreapi.1api.net/api/call.cgi";
	}


	function _domainDetails() {
		$response = $this->_call(array(
			'COMMAND' => "StatusDomain",
			'DOMAIN' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
		));

		if ($response['CODE'] != 200) {
			return false;
		}

		return $response['PROPERTY'];
	}


	/**
	 * Convert the response string as a hash
	 * 
	 * @param string $response
	 * @return array The response as a hash
	 */
	protected function _response_to_hash ($response) {
		if (is_array($response)) return $response;
		$hash = array(
				"PROPERTY" => array(),
				"CODE" => "423",
				"DESCRIPTION" => "Empty response from API"
		);
		if (!$response) return $hash;
		$rlist = explode( "\n", $response );
		foreach ( $rlist as $item ) {
			if ( preg_match("/^([^\=]*[^\t\= ])[\t ]*=[\t ]*(.*)$/", $item, $m) ) {
				$attr = $m[1];
				$value = $m[2];
				$value = preg_replace( "/[\t ]*$/", "", $value );
				if ( preg_match( "/^property\[([^\]]*)\]/i", $attr, $m) ) {
					$prop = strtoupper($m[1]);
					$prop = preg_replace( "/\s/", "", $prop );
					if ( in_array($prop, array_keys($hash["PROPERTY"])) ) {
						array_push($hash["PROPERTY"][$prop], $value);
					}
					else {
						$hash["PROPERTY"][$prop] = array($value);
					}
				}
				else {
					$hash[strtoupper($attr)] = $value;
				}
			}
		}
		if ( (!$hash["CODE"]) || (!$hash["DESCRIPTION"]) ) {
			$hash = array(
					"PROPERTY" => array(),
					"CODE" => "423",
					"DESCRIPTION" => "Invalid response from API"
			);
		}
		return $hash;
	}

	/**
	 * Encode the command array in a command-string
	 *
	 * @param array $commandarray Command array
	 * @return string Encoded command as a string
	 */
	protected function _command_encode($commandarray) {
		if (!is_array($commandarray)) return $commandarray;
		$command = "";
		foreach ( $commandarray as $k => $v ) {
			if ( is_array($v) ) {
				$v = $this->_command_encode($v);
				$l = explode("\n", trim($v));
				foreach ( $l as $line ) {
					$command .= "$k$line\n";
				}
			}
			else {
				$v = preg_replace( "/\r|\n/", "", $v );
				$command .= "$k=$v\n";
			}
		}
		return $command;
	}

	/**
	 * Make a curl API call over HTTP(S) and returns the response as a string
	 * 
	 * @param array $command The command array
	 * @param array|string $config The config array
	 */
	protected function _call_raw_http($command, $config=null){
		$c = new Curl();

		$c->setURL($this->getURL());
		$c->setFieldMethod(1); // get method

		if ( $this->config['moduleTestMode'] ) {
			$c->addField('s_entity', '1234');
		}
		else {
			$c->addField('s_entity', '54cd');
		}

		$c->addField('s_login', $this->UID);
		$c->addField('s_pw', $this->PW);
		$c->addField('s_command', $command);

		return $c->toString();
	}
	
	/**
	 * Make a curl API call and returns the response as a string 
	 * 
	 * @param array $command The command array
	 * @param array|string $config The config array or if given as string, the s_user
	 */
	protected function _call_raw($command, $config=null){
		return $this->_call_raw_http($command, $config);
	}
	
	/**
	 * Make a curl API call and returns the response as a response object, in the 
	 * case of a exception, raise it.
	 * 
	 * @param array $command The command array
	 * @param array|string $config The config array or if given as string, the s_user
	 */
	protected function _call($command, $config=null, $exception=true){
		$command = $this->_command_encode($command);
		$response = $this->_call_raw($command, $config);

		$hash = $this->_response_to_hash($response);
		
		if ( $exception && !preg_match('/^2/', $hash["CODE"]) ) {
			$this->_call_errors[] = $hash["CODE"]." ".$hash["DESCRIPTION"]."\n\n[COMMAND]\n".htmlentities($command, ENT_QUOTES, "UTF-8")."\n\n".htmlentities($response, ENT_QUOTES, "UTF-8")."\n";
		}

		return $hash;
	}
	


// ---------------------------------------------------------------------------------------------------- REGISTRAR METHODS
// ---------------------------------------------------------------------------------------------------- REGISTRAR METHODS

	
	function getRegistrantParams($action, $domain_sld, $tld, $contact = NULL, $withTLDParams = false) {
		$originalparams = parent::getRegistrantParams($action, $domain_sld, $tld, $contact, $withTLDParams);
		
		//display additional domain fields
		//####################################################################
		$domain_tld = $tld->tld_extension;

		include 'hexonet_additionalfields.php';
		
		if(isset($additionaldomainfields[".".$domain_tld])){
			$tld_additional_fields = $additionaldomainfields[".".$domain_tld];
		}
		
		if(isset($tld_additional_fields)){
			foreach($tld_additional_fields as $field){
				$tmp_item = array();
				if(isset($field["Name"])){
					$tmp_item["title"] = $field["Name"];
				}
				if(isset($field["Ispapi-Name"])){
					$tmp_item["id"] = $domain_sld."_".strtolower($field["Ispapi-Name"]);
				}
				if(isset($field["Type"])){
					if($field["Type"] == "dropdown"){
						$type = "s";
					}
					if($field["Type"] == "tickbox"){
						$type = "b";
					}
					if($field["Type"] == "text"){
						$type = "t";
					}
					if($field["Type"] == "radiobox"){
						$type = "r";
					}
					$tmp_item["type"] = $type;
				}
				if(isset($field["Ispapi-Options"]) && isset($field["Options"])){
					$ispapi_option = explode("|", $field["Ispapi-Options"]);
					$options = explode("|", $field["Options"]);
					
					$i=0;
					foreach($ispapi_option as $option){
						$list = array($option, $options[$i]);
						$tmp_item["items"][] = $list;
						$i++;
					}
					
					if(isset($field["Default"])){
						$key = array_search($field["Default"], $ispapi_option);
						if(isset($ispapi_option[$key])){
							$tmp_item["default"] = $ispapi_option[$key];
						}
					}
				}
				if(isset($field["Required"])){
					if($field["Required"]){
						$tmp_item["required"] = "1";
					}else{
						$tmp_item["required"] = "0";
					}
				}
				if(isset($field["Description"])){
					$tmp_item["description"] = $field["Description"];
				}
				
				$tmp_item["value"] = "";
				$tmp_item["hidden"] = "";
				$tmp_item["read_only"] = 0;
				$tmp_item["validators"] = "";
				
				$originalparams[1]["inputs"][] = $tmp_item;
			}
		}
		//####################################################################
		
		return $originalparams;
	}


	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {

		$command = array(
			'COMMAND' => 'AddDomain',
			'DOMAIN' => $this->input["domainSLD"].".".$this->input["domainTLD"],
			'PERIOD' => $this->input["domainYears"],
			'NAMESERVER' => array(),
			'OWNERCONTACT' => array(),
			'ADMINCONTACT' => array(),
			'TECHCONTACT' => array(),
			'BILLINGCONTACT' => array()
		);

		$this->arrayData['OtherData'] = array(
			"input" => $this->input,
		);

		foreach ( $this->contactTypesMap as $type => $rtype ) {
			$command[$rtype][0] = array();
			if($this->input["config"]["{$type}FirstName"] || $this->input["config"]["{$type}LastName"] || $this->input["config"]["{$type}OrganizationName"]) {
				foreach ( $this->contactParamsMap as $param => $rparam ) {
					$command[$rtype][0][$rparam] = html_entity_decode($this->input["config"]["${type}${param}"], ENT_QUOTES);
				}
			}
			else {
				foreach ( $this->contactParamsMap as $param => $rparam ) {
					$command[$rtype][0][$rparam] = html_entity_decode($this->input["registrant${param}"], ENT_QUOTES);
				}
			}

			if ( $command[$rtype][0]["PHONE"] ) {
				$command[$rtype][0]["PHONE"] = preg_replace('/^\+/', '', $command[$rtype][0]["PHONE"]);
				$command[$rtype][0]["PHONE"] = preg_replace('/\./', '', $command[$rtype][0]["PHONE"]);
			}
			if ( $command[$rtype][0]["FAX"] ) {
				$command[$rtype][0]["FAX"] = preg_replace('/^\+/', '', $command[$rtype][0]["FAX"]);
				$command[$rtype][0]["FAX"] = preg_replace('/\./', '', $command[$rtype][0]["FAX"]);
			}
		}

		if ($this->config['moduleUseCustomNameserver'] && $this->input["domainNameserver1"]) {
			for ($i=1; $i<=13; $i++ ) {
				if (notNull( $this->input["domainNameserver$i"] )) {
					$command['NAMESERVER'][] = $this->input["domainNameserver$i"];
				}
			}
		}
		else {
			for ($i=1; $i<=13; $i++ ) {
				if (notNull( $this->input["config"]["domainNameserver$i"] )) {
					$command['NAMESERVER'][] = $this->input["config"]["domainNameserver$i"];
				}
			}
		}

		if ($this->input['domainRegistrarLock']) {
			$params['TRANSFERLOCK'] = 1;
		}
		
		//check and add additional domain fields to the register command
		//####################################################################
		include 'hexonet_additionalfields.php';
		
		if(isset($additionaldomainfields[".".$this->input["domainTLD"]])){
			$tld_additional_fields = $additionaldomainfields[".".$this->input["domainTLD"]];
		}
		
		if(isset($tld_additional_fields)){
			foreach($tld_additional_fields as $field){
				if(isset($field["Ispapi-Name"])){
					if(isset($this->input[strtolower($field["Ispapi-Name"])])){
						$command[$field["Ispapi-Name"]] = $this->input[strtolower($field["Ispapi-Name"])];
					}
				}
			}
		}
		//$var = print_r($command, true);
		//####################################################################
		
		$response = $this->_call($command);

		if (isset($response['CODE']) && $response['CODE'] == 200) {
			$this->arrayData += array(
				'domainDateCreated' => strtotime($response['PROPERTY']['CREATEDDATE'][0]),
				'domainDateExpires' => strtotime($response['PROPERTY']['REGISTRATIONEXPIRATIONDATE'][0]),
				'registrantContactID' => $response['PROPERTY']['OWNERCONTACT'][0],
				'adminContactID' => $response['PROPERTY']['ADMINCONTACT'][0],
				'auxBillingContactID' => $response['PROPERTY']['BILLINGCONTACT'][0],
				'techContactID' => $response['PROPERTY']['TECHCONTACT'][0],
				'nameservers' => $response['PROPERTY']['NAMESERVER'],
				'domainStatus' => ( $response['PROPERTY']['STATUS'][0] == 'ACTIVE' ? 0 : 1 ),
				'domainName' => $response['PROPERTY']['DOMAIN'][0],
				'domainAuthCode' => $response['PROPERTY']['AUTH'][0],
				'zone' => $response['PROPERTY']['ZONE'][0]
				);
			return $this->_exit(1);
		}
		return $this->_exit(0);
	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
		$response = $this->_call(array(
			'COMMAND' => 'TransferDomain',
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'auth' => $this->input['DomainPassword']
			) );

		switch( $response['CODE'] ) {
		case 200:
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
			break;
		case 531:
			$this->addError(TRANS_MBAPIERROR, "Authorization failed (wrong or missing auth-info code)");
			break;
		default:
			$this->addError(TRANS_MBAPIERROR, "Domain Transfer Failed [$response[DESCRIPTION]]");
			break;
		}

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
		$response = $this->_call(array(
			'COMMAND' => 'RenewDomain',
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'period' => $this->input['NumYears']
			) );
		if ($response['CODE'] == 510) {
			$response = $this->_call(array(
				'COMMAND' => 'PayDomainRenewal',
				'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
				'period' => $this->input['NumYears']
				) );
		}


		if ($response['CODE'] == 200) {
			$this->arrayData["DomainRenewSuccess"] = 1;

			$this->hasExecutedSuccessfully = 1;
			$this->hasExecuted = 1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		} else {
			$this->arrayData["DomainRenewSuccess"] = 0;

			$this->addError(TRANS_REGISTRARERROR, 'Error: '.$response['DESCRIPTION']);
			$this->hasExecutedSuccessfully = 0;
			$this->hasExecuted = 1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}




	/**
	 *	Call the registrar and fetch domain list
	 *
	 *	@return string XML containing domain list
	 */
	function getDomains() {
		global $CONF;

		$params["first"] = (int)$this->input["offset"];

		$params["COMMAND"] = 'QueryDomainList';
		$response = $this->_call($params);

		if (isset($response['PROPERTY']) && $response['PROPERTY']['TOTAL'] > 0) {
			$domainList = $response['PROPERTY']['DOMAIN'];

			$this->arrayData["domains"] = array();

			foreach( (array)$domainList as $domain ) {
				if (notNull($domain)) {

					list($sld, $tld, $tld2) = explode('.', $domain);
					if (notNull($tld2))
						$tld .= ".$tld2";
					unset($tld2);

					$this->arrayData["domains"][] = array(
						"domainName" => $domain,
						"domainSLD" => $sld,
						"domainTLD" => $tld,
						);
				}
			}
		}

		if (isset($response['PROPERTY'])) {
			$this->arrayData["totalDomains"] = $response['PROPERTY']['TOTAL'][0];
			$this->arrayData["domainStartPosition"] = (int)$response['PROPERTY']['FIRST'][0];
			$this->arrayData["domainEndPosition"] = $response['PROPERTY']['LAST'][0];
		}

		return $this->_exit();
	}

	function getDomainDNS() {
		
		if ( ($details = $this->_domainDetails()) == false ) {
			return $this->_exit(0);
		}

		$this->arrayData['listDNS'] = $details['NAMESERVER'];
		if ( !$this->arrayData['listDNS'] ) $this->arrayData['listDNS'] = array();

		while ( count($this->arrayData['listDNS']) < 4 ) {
			$this->arrayData['listDNS'][] = '';
		}

		$this->arrayData["UseDNS"] = (isset($details["INTERNALNAMESERVER"]))? "1" : "0";

		//echo "<pre>";
		//print_r($this->arrayData['listDNS']);

		$dnsOptions = array(
		    'A' => 'A',
			'AAAA' => 'AAAA',
			'CNAME' => 'CNAME',
			'MX' => 'MX',
			'TXT' => 'TXT',
		    'URL' => 'URL Redirect',
			'X-SMTP' => 'MAIL Forward'
		);

		$this->arrayData['dnsOptions'] = $dnsOptions;
		
		$dnszone = $this->input['domainSLD'] .'.'. $this->input['domainTLD'].".";

		$command = array(
				"COMMAND" => "QueryDNSZoneRRList",
				"DNSZONE" => $dnszone,
				"SHORT" => 1,
				"EXTENDED" => 1
		);
		$response = $this->_call($command);
		
		$hostrecords = array();
		
		if ( $response["CODE"] == 200 ) {
			$i = 0;
			foreach ( $response["PROPERTY"]["RR"] as $rr ) {
				$fields = explode(" ", $rr);
				$domain = array_shift($fields);
				$ttl = array_shift($fields);
				$class = array_shift($fields);
				$rrtype = array_shift($fields);
		
				if ( $rrtype == "A" ) {
					$hostrecords[$i] = array( "name" => $domain, "type" => $rrtype, "address" => $fields[0], );
		
					if ( preg_match('/^mxe-host-for-ip-(\d+)-(\d+)-(\d+)-(\d+)$/i', $domain, $m) ) {
						unset($hostrecords[$i]);
						$i--;
					}
					$i++;
				}
		
				if ( $rrtype == "AAAA" ) {
					$hostrecords[$i] = array( "name" => $domain, "type" => "AAAA", "address" => $fields[0], );
					$i++;
				}
		
				if ( $rrtype == "MX" ) {
		
					if ( preg_match('/^mxe-host-for-ip-(\d+)-(\d+)-(\d+)-(\d+)($|\.)/i', $fields[1], $m) ) {
						$hostrecords[$i] = array( "name" => $domain, "type" => "MXE", "address" => $m[1].".".$m[2].".".$m[3].".".$m[4], );
					}
					else {
						$hostrecords[$i] = array( "name" => $domain, "type" => $rrtype, "address" => $fields[1], "priority" => $fields[0] );
					}
					$i++;
				}
		
				if ( $rrtype == "TXT" ) {
					$hostrecords[$i] = array( "name" => $domain, "type" => $rrtype, "address" => implode(" ", $fields), );
					$i++;
				}
		
				if ( $rrtype == "CNAME" ) {
					$hostrecords[$i] = array( "name" => $domain, "type" => $rrtype, "address" => $fields[0], );
					$i++;
				}
		
				if ( $rrtype == "X-HTTP" ) {
					if ( preg_match('/^\//', $fields[0]) ) {
						$domain .= array_shift($fields);
					}
					$url_type = array_shift($fields);
					if ( $url_type == "REDIRECT" ) $url_type = "URL";
		
					$hostrecords[$i] = array( "name" => $domain, "type" => $url_type, "address" => implode(" ",$fields), );
					$i++;
				}
				
				if ( $rrtype == "X-SMTP" ) {
					if($domain != "@"){
						$host = $domain.".".substr($dnszone, 0, -1);
					}else{
						$host = substr($dnszone, 0, -1);
					}
					$from = array_shift($fields).$host;
					$url_type = array_shift($fields);
					if ( $url_type == "MAILFORWARD" ) $url_type = "X-SMTP";
					
					$hostrecords[$i] = array( "name" => $from, "type" => $url_type, "address" => implode(" ",$fields), );
					$i++;
				}
				
			}
		}

		$this->arrayData['host'] = $hostrecords;

		return $this->_exit(1);
	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {
		$domain = $this->input['domainSLD'] .'.'. $this->input['domainTLD'];
		
		$command = array(
			'COMMAND' => 'ModifyDomain',
			'DOMAIN' => $domain,
			'NAMESERVER' => array()
		);

		if ( ($this->input['domainDNS']) || !is_array($this->input["domainNameserver"]) ) {
			for( $i=1; $i<=13; $i++ ) {
				if ( $this->input["domainNameserver$i"] ) {
					$command['NAMESERVER'][] = $this->input["domainNameserver${i}"];
				}
			}
		}
		else {
			for( $i=0; $i<=12; $i++ ) {
				if ( $this->input["domainNameserver"][$i] ) {
					$command['NAMESERVER'][] = $this->input["domainNameserver"][$i];
				}
			}
		}

		$response = $this->_call($command);
		
		
		$dnszone = $this->input['domainSLD'] .'.'. $this->input['domainTLD'].".";
		
		//check if DNS Zone exists, if not, create it.
		$command = array(
				"COMMAND" => "QueryDNSZoneRRList",
				"DNSZONE" => $dnszone
		);
		$response = $this->_call($command, null, false);
		
		if ($response["CODE"]=="545"){
			//create DNS Zone
			$command = array(
					"COMMAND" => "CreateDNSZone",
					"DNSZONE" => $dnszone
			);
			$response = $this->_call($command);	
		}
		
		$command = array(
				"COMMAND" => "UpdateDNSZone",
				"DNSZONE" => $dnszone,
				"INCSERIAL" => 1,
				"EXTENDED" => 1,
				"DELRR" => array("% A", "% AAAA", "% CNAME", "% TXT", "% MX", "% X-HTTP", "% X-SMTP"),
				"ADDRR" => array(),
		);
		$mxe_hosts = array();

		$dns = array();
		if(isset($this->input["host"]["HostName"])){
			$dns[] = array("HostName" => $this->input["host"]["HostName"],
						   "RecordType" => strtoupper($this->input["host"]["RecordType"]),
						   "Address" => $this->input["host"]["Address"]);
		}else{
			$dns = $this->input["host"];
		}
		
		foreach ($dns as $values) {
			$hostname = $values["HostName"];
			$type = strtoupper($values["RecordType"]);
			$address = $values["Address"];
			$priority = "";
		
			if ( strlen($hostname) && strlen($address) ) {
				if ( $type == "A" ) {
					if ( preg_match('/:/', $address ) ) {
						$type = "AAAA";
					}
					$command["ADDRR"][] = "$hostname $type $address";
				}
				if ( $type == "AAAA" ) {
					$command["ADDRR"][] = "$hostname $type $address";
				}
				if ( $type == "CNAME" ) {
					$command["ADDRR"][] = "$hostname $type $address";
				}
				if ( $type == "TXT" ) {
					$command["ADDRR"][] = "$hostname $type $address";
				}
				if ( $type == "MXE" ) {
					$mxpref = 100;
					if ( preg_match('/^([0-9]+) (.*)$/', $address, $m ) ) {
						$mxpref = $m[1];
						$address = $m[2];
					}
					if ( preg_match('/^([0-9]+)$/', $priority) ) {
						$mxpref = $priority;
					}
		
					if ( preg_match('/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/', $address, $m) ) {
						$mxe_host = "mxe-host-for-ip-$m[1]-$m[2]-$m[3]-$m[4]";
						$ip = $m[1].".".$m[2].".".$m[3].".".$m[4];
						$mxe_hosts[$ip] = $mxe_host;
						$command["ADDRR"][] = "$hostname MX $mxpref $mxe_host";
					}
					else {
						$address = "$mxpref $address";
						$type = "MX";
					}
				}
				if ( $type == "MX" ) {
					$mxpref = 100;
					if ( preg_match('/^([0-9]+) (.*)$/', $address, $m ) ) {
						$mxpref = $m[1];
						$address = $m[2];
					}
					if ( preg_match('/^([0-9]+)$/', $priority) ) {
						$mxpref = $priority;
					}
		
					$command["ADDRR"][] = "$hostname $type $mxpref $address";
				}
				if ( $type == "FRAME" ) {
					$redirect = "FRAME";
					if ( preg_match('/^([^\/]+)(.*)$/', $hostname, $m) ) {
						$hostname = $m[1];
						$redirect = $m[2]." ".$redirect;
					}
					$command["ADDRR"][] = "$hostname X-HTTP $redirect $address";
				}
				if ( $type == "URL" ) {
					$redirect = "REDIRECT";
					if ( preg_match('/^([^\/]+)(.*)$/', $hostname, $m) ) {
						$hostname = $m[1];
						$redirect = $m[2]." ".$redirect;
					}
					$command["ADDRR"][] = "$hostname X-HTTP $redirect $address";
				}
				if ( $type == "X-SMTP" ) {
					$domain = "";
					$name = "";
					$redirect = "MAILFORWARD";
					$arr = preg_split("/@/", $hostname);
					$nbocc = count($arr);
					if($nbocc==2){
						if(empty($arr[0]) && empty($arr[1])){
							$name = "@";
						}else{
							$name = $arr[0]."@";
							$domain = $arr[1];
						}
					}else{
						$name = $arr[0]."@";
					}
					
					$prefix = "";
					if(!empty($domain)){
						$orighostname = $this->input["domainSLD"].".".$this->input["domainTLD"];
						$arr = preg_split("/".$orighostname."/", $domain);
						if(!empty($arr[0])){
							$prefix = substr($arr[0],0,-1);
						}
					}
					if(empty($prefix)){
						$prefix = "@";
					}

					$command["ADDRR"][] = $prefix." X-SMTP $name $redirect $address";
				}
			}
		}
		foreach ( $mxe_hosts as $address => $hostname ) {
			$command["ADDRR"][] = "$hostname A $address";
		}	
		//echo "<pre>";	
		//print_r($command);
		$response = $this->_call($command);
		//die();
		if ( !($response['CODE'] == 200) ) {
			$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
			return $this->_exit(0);
		}

		$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
		return $this->_exit(1);
	}


	/**
	 * Call the registrar to fetch a domain's contact data
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		if (notNull($this->input['domainSLD'])) {
			$this->arrayData += array(
				'DomainData' => array(
					'sld' => $this->input['domainSLD'],
					'tld' => $this->input['domainTLD'],
					'domainName' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
					) );
		}

		$contactTrans = array(
			'ADMINCONTACT' => 'Admin',
			'TECHCONTACT' => 'Tech',
			'BILLINGCONTACT' => 'AuxBilling',
			'OWNERCONTACT' => 'Registrant');

		foreach( $contactTrans as $rType => $mb ) {
			$response = $this->_call(array('COMMAND' => "StatusContact", 'contact' => $details[ $rType ][0]));

			$this->arrayData += array(
				"${mb}ContactData" => array(
					"${mb}FirstName" => $response['PROPERTY']['FIRSTNAME'][0],
					"${mb}LastName" => $response['PROPERTY']['LASTNAME'][0],
					"${mb}OrganizationName" => $response['PROPERTY']['ORGANIZATION'][0],
					"${mb}EmailAddress" => $response['PROPERTY']['EMAIL'][0],
					"${mb}Address1" => $response['PROPERTY']['STREET'][0],
					"${mb}Address2" => $response['PROPERTY']['STREET'][1],
					"${mb}City" => $response['PROPERTY']['CITY'][0],
					"${mb}StateProvince" => $response['PROPERTY']['STATE'][0],
					"${mb}PostalCode" => $response['PROPERTY']['ZIP'][0],
					"${mb}Country" => $response['PROPERTY']['COUNTRY'][0],
					"${mb}Phone" => $response['PROPERTY']['PHONE'][0],
					"${mb}Fax" => $response['PROPERTY']['FAX'][0]
					) );

		}
		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();
	} // END getDomainContactData()

	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function _stdPhoneNumber($country, $number, $ext) {
		if ( ($number === NULL) || !strlen($number) ) {
			return '';
		}

		if ( ($ext !== NULL) && strlen($ext) ) {
			$number .= "x".$ext;
		}
		if ( $country ) {
//			$number = "+$country.$number";
		}
		return $number;
	}


	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData() {
		$modifyDomainParams = array(
			'COMMAND' => "ModifyDomain",
			'DOMAIN' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
		);

		foreach( (array)$this->input['domainContactData'] as $type => $contact ) {
			$rType = $this->contactTypesMap[$type];

			$phone = $this->_stdPhoneNumber($contact["${type}PhoneDial"], $contact["${type}Phone"], $contact["${type}PhoneExt"]);
			$fax = $this->_stdPhoneNumber($contact["${type}FaxDial"], $contact["${type}Fax"], $contact["${type}FaxExt"]);


			$modifyDomainParams += array(
				"${rType}0title" => $contact["${type}JobTitle"],
				"${rType}0firstname" => $contact["${type}FirstName"],
				"${rType}0lastname" => $contact["${type}LastName"],
				"${rType}0organization" => $contact["${type}OrganizationName"],
				"${rType}0street0" => $contact["${type}Address1"],
				"${rType}0street1" => $contact["${type}Address2"],
				"${rType}0city" => $contact["${type}City"],
				"${rType}0state" => $contact["${type}StateProvince"],
				"${rType}0zip" => $contact["${type}PostalCode"],
				"${rType}0country" => $contact["${type}Country"],
				"${rType}0phone" => $phone,
				"${rType}0fax" => $fax,
				"${rType}0email" => $contact["${type}EmailAddress"],
				);
		}

		$response = $this->_call($modifyDomainParams);

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
			return $this->_exit(1);
		} else {
			$this->arrayData['result'] = "TRANS_FAILURE";
			return $this->_exit(0);
		}
	}

	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setdomainRenewTypeOptions() {
		$response = $this->_call(array(
			'COMMAND' => "SetDomainRenewalMode",
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'renewalMode' => ($this->input['domainRenewType']=='1'?"AUTORENEW":"AUTOEXPIRE")
			) );

		if ($response['CODE']== 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
		} else {
			$this->arrayData['result'] = "TRANS_FAILURE";
		}

		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for fetching the domain domain autorenew options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainRenewTypeOptions() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData['domainRenewType'] = 0;
		if (mb_eregi("autorenew", $details['RENEWALMODE'][0])) {
			$this->arrayData['domainRenewType'] = 1;
		}
		$this->hasExecutedSuccessfully = 1;


		return $this->_exit();

	}

	/**
	 * This method contains the registrar specific code for fetching the domain expiration date.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExpDate() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData += array(
			'DomainExpDate' => $details['REGISTRATIONEXPIRATIONDATE'][0],
			'DomainExpTimestamp' => strtotime($details['REGISTRATIONEXPIRATIONDATE'][0])
			);

		return $this->_exit();

	}

	function getDomainExtendedInfo() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData += array(
			'DomainExpDate' => $details['REGISTRATIONEXPIRATIONDATE'][0],
			'DomainExpTimestamp' => strtotime($details['REGISTRATIONEXPIRATIONDATE'][0]),
			'DomainRenewDefaultYears' => 1,
			'DomainRenewMaxYears' => 1,
			'DomainRenewPrice' => 0
			);
		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();

	}


	/*
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		$response = $this->_call(array(
			'COMMAND' => "StatusDomainTransfer",
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		if ($response['CODE']== 200) {
			$this->arrayData = $response['TRANSFERSTATUS'][0];
			$this->hasExecutedSuccessfully = 1;
		}

		return $this->_exit();
	}

	function checkDomainAvailability() {
		$response = $this->_call(array(
			'COMMAND' => 'CheckDomain',
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		switch ($response['CODE'][0]) {
		/// is available
		case 210:$this->arrayData['OriginalDomainIsAvailable'] = 1;break;

		/// not available
		case 211:$this->arrayData['OriginalDomainIsAvailable'] = 0;break;

		/// unknown
		default:
			$this->arrayData += array(
				'OriginalDomainIsAvailable' => 0,
				'OriginalDomainAvailabilityUnknown' => 1
				);
			break;
		}
		
		return $this->_exit();
	}

	function checkDomainTransferAvailability() {
		$response = $this->_call(array(
			'COMMAND' => "CheckDomainTransfer",
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		switch ($response['CODE'][0]) {
		/// is available
		case 218:
			$this->arrayData += array(
				'result' => "TRANS_AVAILABLE",
				'DomainName' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
				'DomainRegistrar' => $response['PROPERTY']['REGISTRAR'][0],
				'OtherData' => array(
					'whoisServer' => $response['PROPERTY']['WHOISSERVER'][0],
					'domainDateCreated' => strtotime($response['PROPERTY']['CREATEDDATE'][0]),
					'domainAgeInDays' => $response['PROPERTY']['AGEINDAYS'][0],
					'transferLock' => $response['PROPERTY']['TRANSFERLOCK'][0],
					'adminContactName' => $response['PROPERTY']['ADMINCONTACTNAME'][0],
					'adminContactEmail' => $response['PROPERTY']['ADMINCONTACTEMAIL'][0],
					'adminContactLang' => $response['PROPERTY']['ADMINCONTACTLANGUAGE'][0]
					)
				);
			break;

		/// not available
		case 219:
			$this->arrayData['result'] = "TRANS_NOT_AVAILABLE";
			break;

		/// unknown
		default:
			$this->arrayData['result'] = "TRANS_FAILURE";
			break;
		}

		return $this->_exit();
	}

	protected function _exit($hasExecutedSuccessfully = NULL) {
		if ( $hasExecutedSuccessfully === NULL ) {
		}
		elseif ( $hasExecutedSuccessfully ) {
			$this->hasExecuted = 1;
			$this->hasExecutedSuccessfully = 1;
		}
		else {
			$this->hasExecutedSuccessfully = 0;
			foreach ( $this->_call_errors as $error ) {
				$this->addError(TRANS_REGISTRARERROR, $error);
			}
		}
		$this->_call_errors = array();
		return $this->getRegistrarReturnXML();
	}


	function getDomainAccessPassword() {
		if ( ($details = $this->_domainDetails()) == false ) {
			return $this->_exit(0);
		}

		$this->arrayData += array(
			'DomainAccessPasswordSet' => 1,
			'DomainAccessPassword' => $details['AUTH'][0]
			);
		return $this->_exit(1);
	}

	function setDomainAccessPassword() {
		$response = $this->_call(array(
			'COMMAND' => "ModifyDomain",
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'auth' => $this->input['domainAccessPassword']
			) );

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->arrayData['result'] = "TRANS_FAILED";
			$this->addError(TRANS_MBAPIERROR, "Updating AuthInfo Failed [$response[DESCRIPTION]]");
			$this->hasExecutedSuccessfully = 0;
		}
	
		return $this->_exit();
	}


	function getDomainRegistrarLockOptions() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		if ( isset($details["TRANSFERLOCK"]) ) {
			$this->arrayData['DomainIsLockable'] = 1;
			$this->arrayData['DomainRegistrarLock'] = $details["TRANSFERLOCK"][0];
		}
		else {
			$this->arrayData['DomainIsLockable'] = 0;
		}

		return $this->_exit();
	}

	function setDomainRegistrarLockOptions() {
		$params = array('domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']);

		if ($this->input['domainRegistrarLock']) {
			$params['TRANSFERLOCK'] = 1;
		} else {
			$params['TRANSFERLOCK'] = 0;
		}

		$params['COMMAND'] = "ModifyDomain";
		$response = $this->_call($params);

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = $response['DESCRIPTION'];
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->addError(TRANS_MBAPIERROR, "Updating Registrar Lock Option Failed [$response[DESCRIPTION]]");
		}

		return $this->_exit();
	}


// ---------------------------------------------------------------------------------------------------- CONFIG PARAMS
// ---------------------------------------------------------------------------------------------------- CONFIG PARAMS

	/**
	 * Returns the default configuration parameters for the ISPAPI module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {

		$defaultConfigParams = array(

			// admin contact data
			array(NULL, NULL, "adminFirstName", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,18,0, NULL),
			array(NULL, NULL, "adminLastName", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,19,0, NULL),
			array(NULL, NULL, "adminOrganizationName", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,20,0, NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,21,0, NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,22,0, NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,23,0, NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,24,0, NULL),
			array(NULL, NULL, "adminState", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,25,0, NULL),
			array(NULL, NULL, "adminZip", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,26,0, NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,27,0, NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,28,0, NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_REGISTRAR_ADMIN_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,29,0, NULL),
			// tech contact data
			array(NULL, NULL, "techFirstName", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,30,0, NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,31,0, NULL),
			array(NULL, NULL, "techOrganizationName", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,32,0, NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,33,0, NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,34,0, NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,35,0, NULL),
			array(NULL, NULL, "techCity", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,36,0, NULL),
			array(NULL, NULL, "techState", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,37,0, NULL),
			array(NULL, NULL, "techZip", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,38,0, NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,39,0, NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,40,0, NULL),
			array(NULL, NULL, "techFax", "", "TRANS_REGISTRAR_TECH_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,41,0, NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstName", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,42,0, NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,43,0, NULL),
			array(NULL, NULL, "auxBillingOrganizationName", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,44,0, NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,45,0, NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,46,0, NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,47,0, NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,48,0, NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,49,0, NULL),
			array(NULL, NULL, "auxBillingZip", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,50,0, NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,51,0, NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,52,0, NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_REGISTRAR_AUXBILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,53,0, NULL),
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,54,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,55,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,56,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,57,0, array())

		); // $defaultConfigParams = array(

		$adminOptions = array(
			array('moduleUsername', 't', NULL, 'USERNAME', NULL, NULL, 1),
			array('modulePassword', 'p', NULL, 'PASSWORD', NULL, NULL, 1),
			array('moduleTestMode', 'b', '1', 'ENABLE_TEST_MODE', NULL, NULL, 0),
			array('moduleAllowAutoRenew', 'r', '0', 'REGISTRARSHELL_ALLOW_AUTORENEW', NULL, array(
					array(NULL, NULL, '0', 'TRANS_OBEY_TLD', 1),
					array(NULL,	NULL, '1', 'TRANS_REGISTRAR_AUTORENEW_OPTION', 2),
					array(NULL, NULL, '2', 'TRANS_MODERNBILL_RENEW_OPTION', 3),
			), 0),
			array('moduleUseCustomNameserver', 'b', '0', 'REGISTRARSHELL_USE_CUSTOM_NAMESERVER', 'ISPPROXY_USE_CUSTOM_NAMESERVER_DESC', NULL, 0),
			array('moduleAllowChangeDNS', 'b', '0', 'REGISTRARSHELL_ALLOW_CHANGE_DNS', 'ISPPROXY_ALLOW_CHANGE_DNS_DESC', NULL, 0),
			array('moduleAllowChangeContactInfo', 'b', '1', 'REGISTRARSHELL_ALLOW_CHANGE_CONTACT_INFO', 'ISPPROXY_ALLOW_CHANGE_CONTACT_INFO_DESC', NULL, 0),
			array('moduleAllowChangeRegistrarLock', 'b', '0', 'REGISTRARSHELL_ALLOW_CHANGE_REGISTRAR_LOCK', 'REGISTRARSHELL_ALLOW_CHANGE_REGISTRAR_LOCK_DESC', NULL, 0),
			array('moduleAllowChangeAutorenew', 'b', '0', 'REGISTRARSHELL_ALLOW_CHANGE_AUTORENEW', 'ISPPROXY_ALLOW_CHANGE_AUTORENEW_DESC', NULL, 0),
			array('moduleAllowChangeUpdateDomainPassword', 'b', '0', 'REGISTRARSHELL_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD', 'REGISTRARSHELL_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD', NULL, 0),

		);

        $i = 1;
		$moduleConfig = array();
        foreach($adminOptions as $adminOption) {
            $moduleConfig[] = array(
                NULL,
                NULL,
                $adminOption[0],
                $adminOption[2],
                'TRANS_REGISTRAR_MODULE_SETTINGS',
                'TRANS_REGISTRAR_'.$adminOption[3],
                (is_null($adminOption[4]) ? '' : 'TRANS_REGISTRAR_'.$adminOption[4]),
                $adminOption[1],
                $adminOption[6],
                $i++,
                0,
                ($adminOption[1] == 'r')
                    ? (($adminOption[5] != NULL) ? $adminOption[5] : array(
                        array(NULL,NULL,'1','TRANS_YES',2),
                        array(NULL,NULL,'0','TRANS_NO',1)
                    ))
                    : NULL
            );
        }

		$moduleConfig = array_merge($moduleConfig, $defaultConfigParams);
		return array_merge($this->configParams,$moduleConfig);

		// Grab the configs set at the registrar level
		//return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()

} // ISPAPI

?>
